//=============================================================================
// Mano_CommandErase.js
// ----------------------------------------------------------------------------
// Copyright (c) 2021-2021 Sigureya
// This software is released under the MIT License.
// http://opensource.org/licenses/mit-license.php
// ----------------------------------------------------------------------------
// Version
// ver 0.9.0 2021/03/08
// ----------------------------------------------------------------------------
// [Twitter]: https://twitter.com/Sigureya/
//=============================================================================

/*:
 * @plugindesc メニュー画面から一部の項目を消します
 * @author しぐれん(https://github.com/Sigureya/RPGmakerMV)
 * @url https://raw.githubusercontent.com/Sigureya/RPGmakerMZ/master/Mano_InputConfig.js
 * 
 * @target MZ
 * 
 * @param Menu
 * @text メニュー
 * @type struct<Menu>
 * @default {"options":"false","gameEnd":"true"}
 * 
 * @param ItemCategory
 * @text アイテムカテゴリ
 * @type struct<ItemCategory>
 * @default {"item":"true","weapon":"true","armor":"true","keyItem":"true"}
 * 
 * @param Equip
 * @text 装備
 * @type struct<Equip>
 * @default {"equip":"true","optimize":"true","clear":"true"}
 * 
 * 
 * @param Options
 * @text オプション
 * @type struct<Options>
 * @default {"alwaysDash":"true","commandRemember":"true","touchUI":"true"}
 * 
 * @param Shop
 * @text ショップ
 * @type struct<Shop>
 * @default {"buy":"true","sell":"true"}
 * 
 * @param Actor
 * @text アクターコマンド(バトル)
 * @type struct<Actor>
 * @default {"attack":"true","guard":"true","item":"true"}
 * 
 * @param Title
 * @text タイトル
 * @type struct<Title>
 * @default {"newGame":"true","continue":"true","options":"true"}
 * 
 * @help
 * ゲームによってはツクールの基本要素が不要になります。
 * このプラグインはそれらを非表示にします。
 * 
 * MZでテストを行っていますが、MVでも動くはずです。
*/

/*~struct~Menu:
  * @param options
  * @text オプション
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param gameEnd
  * @text ゲーム終了
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
*/
 /*~struct~ItemCategory:
  * @param item
  * @text アイテム
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param weapon
  * @text 武器
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param armor
  * @text 防具
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param keyItem
  * @text 大事なもの
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
*/

 /*~struct~Equip:
  * @param equip
  * @text 装備
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param optimize
  * @text 最強装備
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param clear
  * @text 全て外す
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * 
*/

 /*~struct~Options:
  * @param alwaysDash
  * @text 常時ダッシュ
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param commandRemember
  * @text コマンド記憶
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param touchUI
  * @text タッチUI
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
*/
 /*~struct~Shop:
  * @param buy
  * @text 購入
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param sell
  * @text 売却
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
*/
 /*~struct~Actor:
  * @param attack
  * @text 攻撃
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param guard
  * @text 防御
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param item
  * @text アイテム
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
*/

 /*~struct~Title:
  * @param newGame
  * @text ニューゲーム
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param continue
  * @text コンティニュー
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
  * @param options
  * @text オプション
  * @type boolean
  * @on 表示する
  * @off 表示しない
  * @default true
  * 
*/

(function(){
    'use strict';

const PLUGIN_NAME='Mano_CommandErase';
function getParam(){ return PluginManager.parameters(PLUGIN_NAME);  }
/**
 * @param {String} windowName 
 * @param {String} paramName 
 * @param {String[]} symbols
 * @returns 
 */
function pair(windowName,paramName,symbols){
    return {
        windowName:windowName,
        paramName:paramName,
        symbols:symbols
    }
}
/**
 * @param {String} windowName 
 * @param {String} symbol 
 * @returns 
 */
function makeKey(windowName,symbol){
    return symbol + windowName;
}

function createSet(){
    /**
     * @type {Set<String>} set 
     */
     const set = new Set();
     const param = getParam();
     const paramList= [
         pair(Window_MenuCommand.name,"Menu",[]),
         pair(Window_ItemCategory.name,"ItemCategory",["item","weapon","armor","keyItem"]),
         pair(Window_EquipCommand.name,"Equip",["equip","optimize","clear"]),
         pair(Window_Options.name,"Options",["alwaysDash","commandRemember","touchUI"]),
         pair(Window_ShopCommand.name,"Shop",["buy","sell"]),
         pair(Window_ActorCommand.name,"Actor",["attack","guard","item"]),
         pair(Window_TitleCommand.name,"Title",["newGame","continue","options"])
     ];

     for (const iterator of paramList) {
         const objText = param[iterator.paramName];
         
         const obj =JSON.parse(objText||"{}");
         for (const symbol of iterator.symbols) {
             if(obj[symbol]==="false"){
                 const key = makeKey(iterator.windowName,symbol);
                 set.add(key);
             }
         }
     }
     return set;
}

class CommandManager_T{
    /**
     * @param {Set<String>} set 
     */
    constructor(set){
        this._set = set;
    }
    /**
     * @param {String} windowName 
     * @param {String} symbol 
     */
    isEnabled(windowName,symbol){
        const key = makeKey(windowName,symbol);
        if( this._set.has(key)){
            return false;
        }
        return true;
    }
}

const CommandManager = new CommandManager_T(createSet());
const Window_Command_addCommand =Window_Command.prototype.addCommand;
Window_Command.prototype.addCommand =function(
    name, symbol, enabled , ext 
){
    const windowName = this.constructor.name;
    if( CommandManager.isEnabled(windowName,symbol)){
        Window_Command_addCommand.call(this,name, symbol, enabled , ext );
    }
}

}());
